/* You can edit the text in this window to change the
 * appearance of this Window.
 * Be careful, if you screw it up, nothing might be visible
 * anymore. :)
 */

/* The content of reset.css has been applied to reset all properties to
 * their defaults values and overrides all user settings and the theme in use */

/* Set a very futuristic style by default */
* {
  color: green;
  font-family: Monospace;
  border: 1px solid;
}

/* Make sure selections are visible */
:selected {
  background-color: darkGreen;
  color: black;
}
