/* Apply this CSS to get the default values for every property.
 * This is useful when writing special CSS tests that should not be
 * inluenced by themes - not even the default ones.
 * Keep in mind that the output will be very ugly and not look like
 * anything GTK.
 */

* {
  color: inherit;
  font-size: inherit;
  background-color: initial;
  font-family: inherit;
  font-style: inherit;
  font-variant: inherit;
  font-weight: inherit;
  text-shadow: inherit;
  icon-shadow: inherit;
  box-shadow: initial;
  margin-top: initial;
  margin-left: initial;
  margin-bottom: initial;
  margin-right: initial;
  padding-top: initial;
  padding-left: initial;
  padding-bottom: initial;
  padding-right: initial;
  border-top-style: initial;
  border-top-width: initial;
  border-left-style: initial;
  border-left-width: initial;
  border-bottom-style: initial;
  border-bottom-width: initial;
  border-right-style: initial;
  border-right-width: initial;
  border-top-left-radius: initial;
  border-top-right-radius: initial;
  border-bottom-right-radius: initial;
  border-bottom-left-radius: initial;
  outline-style: initial;
  outline-width: initial;
  outline-offset: initial;
  background-clip: initial;
  background-origin: initial;
  background-size: initial;
  background-position: initial;
  border-top-color: initial;
  border-right-color: initial;
  border-bottom-color: initial;
  border-left-color: initial;
  outline-color:  initial;
  background-repeat: initial;
  background-image: initial;
  border-image-source: initial;
  border-image-repeat: initial;
  border-image-slice: initial;
  border-image-width: initial;
  transition-property: initial;
  transition-duration: initial;
  transition-timing-function: initial;
  transition-delay: initial;
  engine: initial;
  gtk-key-bindings: initial;

  -GtkWidget-focus-line-width: 0;
  -GtkWidget-focus-padding: 0;
  -GtkNotebook-initial-gap: 0;
}
